
/*
 *      file    name            file.c
 *      (C) Copyright           Akito Monden    1996 - 1998
 */

/*
 * file.c $B$K$O!$0J2<$N;vJA$K4X$9$k4X?t$,F~$C$F$$$k!%(B
 * $B!&%U%!%$%k$N%*!<%W%s(B/$B%/%m!<%:(B
 * $B!&%a%b%j$N3NJ](B/$B2rJ|(B
 * $B!&%a%b%j$X$N%"%/%;%9(B
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "win.h"
#include "file.h"

#ifdef GCC
#include <unistd.h>
#include <sys/types.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <fcntl.h>
#endif

#ifdef WINDOWS
#include <malloc.h>
#include <io.h>
#include <fcntl.h>
#endif

#ifdef BCC
#include <alloc.h> /* include if TC or BC */
#include <fcntl.h> /* for fopen, O_RDONLY */
#include <sys\stat.h> /* for pmode(S_IREAD, S_IWRITE), creat */
#include <io.h> /* for filelength, creat */
#endif

#ifdef GCC
long filelength(int fd)
{
    struct stat sb;
    fstat(fd, &sb);
    return sb.st_size;
}
#endif

int getc2(file *fp)
{
    unsigned char i;
    if(fp->cur_adr == fp->end_adr) return -1;
    i = (unsigned char)fp->cur_adr[0];
    fp->cur_adr++;
    return (int)i;
}

void fseek2(file *fp, long l, int s)
{
    if(fp->cur_adr > fp->end_adr) fp->end_adr = fp->cur_adr;
    switch(s){
    case SEEK_CUR:
        fp->cur_adr += l;
        break;
    case SEEK_SET:
        fp->cur_adr = fp->top_adr + l;
    }
    if(fp->cur_adr > fp->end_adr) fp->end_adr = fp->cur_adr;
}

file *fopen2(char c1[], char c2[])
{
    long l;
    file *fp = (file *)malloc(sizeof(file));
    if(fp == NULL) return NULL; /* $B0[>o=*N;(B */
    
    /* printf("$B%U%!%$%kL>$O(B %s $B$G$9(B\n",c1); */
    if(c2[0] == 'r'){
#ifdef WINDOWS
        fp->f=CreateFile(c1,GENERIC_READ,0,NULL,
                         OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
        if(fp->f == INVALID_HANDLE_VALUE){
			fclose2(fp);
		    free(fp);
            return NULL; /* $B0[>o=*N;(B */
	}
#else
        fp->f = open(c1,O_RDONLY);
        if(fp->f == -1){
        	fclose2(fp);
		    free(fp);
            return NULL; /* $B0[>o=*N;(B */
        }
#endif
        fp->flag = 0;
#ifdef WINDOWS
        l = GetFileSize(fp->f,NULL);
#else
        l = filelength(fp->f);
#endif
        fp->top_adr = fp->cur_adr = (char *)malloc(l);
        if(fp->top_adr == NULL){
	    free(fp);
            return NULL; /* $B0[>o=*N;(B */
        }
#ifdef WINDOWS
        {
            DWORD size;
            ReadFile(fp->f,fp->top_adr,l,&size,NULL);
        }
#else
        (void)read(fp->f,fp->top_adr,l);
#endif
        fp->end_adr = fp->top_adr + l;
    }else{
#ifdef WINDOWS
        fp->f=CreateFile(c1,GENERIC_WRITE,0,NULL,
                         CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL);
        if(fp->f == INVALID_HANDLE_VALUE){
	    free(fp);
            return NULL; /* $B0[>o=*N;(B */
        }
#else
#ifdef BCC
        fp->f = creat(c1,S_IREAD | S_IWRITE);
#else
        fp->f = creat(c1,0666);
#endif
        if(fp->f == -1){
	    free(fp);
            return NULL; /* $B0[>o=*N;(B */
        }
#endif
        fp->flag = 1;
	/* $B$I$&$;%F%s%]%^%C%W$7$+=q$+$J$$$N$@$+$i!$$"$^$j$?$/$5$s(B
	   $B%a%b%j$r3NJ]$7$J$/$F$b$h$$!%(B */
#ifdef BCC
        fp->top_adr = fp->cur_adr = (char *)malloc(5000);
#else
        fp->top_adr = fp->cur_adr = (char *)malloc(5000);
#endif
	if(fp->top_adr == NULL){
	    free(fp);
	    return NULL; /* $B0[>o=*N;(B */
	}
        fp->end_adr = fp->top_adr;
    }
    return fp; /* $B@5>o=*N;(B */
}

void fwrite2(char *adr, long l, file *fp)
{
    long l1;
    for(l1 = 0; l1 < l; l1++){
        fp->cur_adr[l1] = adr[l1];
    }
    fp->cur_adr += l;
    if(fp->cur_adr > fp->end_adr) fp->end_adr = fp->cur_adr;
}

void fclose2(file *fp)
{
    if(fp->flag == 1){
        if(fp->cur_adr > fp->end_adr) fp->end_adr = fp->cur_adr;
#ifdef WINDOWS
        {
            DWORD size;
            WriteFile(fp->f,fp->top_adr,
                      (long)(fp->end_adr - fp->top_adr),&size,NULL);
        }
#else
	(void)write(fp->f,fp->top_adr,(long)(fp->end_adr - fp->top_adr));
#endif
    }
    
#ifdef WINDOWS
    CloseHandle(fp->f);
#else
    close(fp->f);
#endif
    ffree(fp);
}

void fgetpos2(file *fp, fpos_t *l)
{
    l[0] = (long)(fp->cur_adr - fp->top_adr);
}

void fsetpos2(file *fp, fpos_t *l)
{
    if(fp->cur_adr > fp->end_adr) fp->end_adr = fp->cur_adr;
    fp->cur_adr = fp->top_adr + l[0];
}

/* fpa $B$N8e$m$K(B fpb $B$r$/$C$D$1$F(B fpa $B$r%/%m!<%:$9$k(B */
/* fpb $B$O%a%b%j$N2rJ|$b%/%m!<%:$b$7$J$$(B */
void fclose3(file *fpa, file *fpb)
{
    if(fpa->cur_adr > fpa->end_adr) fpa->end_adr = fpa->cur_adr;
    if(fpb->cur_adr > fpb->end_adr) fpb->end_adr = fpb->cur_adr;
#ifdef WINDOWS
    {
        DWORD size;
        WriteFile(fpa->f,fpa->top_adr,
                  (long)(fpa->end_adr - fpa->top_adr),&size,NULL);
        WriteFile(fpa->f,fpb->top_adr,
                  (long)(fpb->end_adr - fpb->top_adr),&size,NULL);
    }
#else
    (void)write(fpa->f,fpa->top_adr,(long)(fpa->end_adr - fpa->top_adr));
    (void)write(fpa->f,fpb->top_adr,(long)(fpb->end_adr - fpb->top_adr));
#endif

#ifdef WINDOWS
    CloseHandle(fpa->f);
#else
    close(fpa->f);
#endif
    ffree(fpa);
}

/* $B%U%!%$%k$r%*!<%W%s$;$:$K(B malloc $B$9$k$@$1(B */
/* malloc $B$K<:GT$9$k$H(B NULL $B$rJV$9(B */
file *fmalloc(void)
{
    long l;
    file *fp = (file *)malloc(sizeof(file));
    if(fp == NULL) return NULL; /* $B0[>o=*N;(B */
    
    /* fp->f = 0; */
    /* fp->flag = 1; */
    
    /* SMF $B$,%9%C%]%jF~$k$0$i$$%a%b%j$r3NJ]$7$?$$(B */
#ifdef BCC
    fp->top_adr = fp->cur_adr = (char *)malloc(65000);
#else
    fp->top_adr = fp->cur_adr = (char *)malloc(420000);
#endif
    if(fp->top_adr == NULL){
	free(fp);
	return NULL; /* $B0[>o=*N;(B */
    }
    fp->end_adr = fp->top_adr;

    return fp; /* $B@5>o=*N;(B */
}

/* fp $B$N%a%b%j$r2rJ|$9$k(B */
void ffree(file *fp)
{
    free(fp->top_adr);
    free(fp);
}

/* fpa $B$NItJ,$K(B fpb $B$NFbMF$r=q$/(B fpa $B$r%/%m!<%:$9$k(B */
/* fpb $B$O%a%b%j$N2rJ|$b%/%m!<%:$b$7$J$$(B */
void fclose4(file *fpa, file *fpb)
{
    if(fpa->cur_adr > fpa->end_adr) fpa->end_adr = fpa->cur_adr;
    if(fpb->cur_adr > fpb->end_adr) fpb->end_adr = fpb->cur_adr;
#ifdef WINDOWS
    {
        DWORD size;
        WriteFile(fpa->f,fpb->top_adr,
                  (long)(fpb->end_adr - fpb->top_adr), &size, NULL);
    }
#else
    (void)write(fpa->f,fpb->top_adr,(long)(fpb->end_adr - fpb->top_adr));
#endif

#ifdef WINDOWS
    CloseHandle(fpa->f);
#else
    close(fpa->f);
#endif
    ffree(fpa);
}

